import xesam
import gobject


class StateChanger (gobject.GObject):
	"""
	Utility class for changing states and emitting notifications across
	the bus.
	"""
	# Number of steps to stay in each state
	STATE_STEPS = 10
	
	# Delay between state changes
	STATE_DELAY = 1000
	
	def __init__ (self, searcher):
		gobject.GObject.__init__ (self)
		self._searcher = searcher
		
		# Keeps track of the state step count
		self._state_count = 0
		
		self._handle = None
		self._state_info = ["IDLE", "0"]
		
		self._searcher.set_property ("state", self._state_info)
	
	def stop (self):
		if self._handle:
			gobject.source_remove (self._handle)
			self._handle = None
	
	def get_state_info (self):
		return self._state_info
	
	def start (self):
		xesam.debug ("Starting state changer %s" % self)
		self._handle = gobject.timeout_add (self.STATE_DELAY, self.update_state)
	
	def update_state (self):
		self._state_info = self.next_state (self._state_info)
		self._searcher.set_property ("state", self._state_info)
		return True
	
	def next_state (self, old_state):
		"""
		Cycle through engine states in a "natural" way
		@param old_state: a [state_str, value_int] pair
		@return : a new state object
		"""	
		state = old_state[0]
		value = int(old_state[1])
		next_state_v = state
	
		if self._state_count >= self.STATE_STEPS:
			self._state_count = 0
			next_state_v = xesam.STATES[(xesam.STATES.index(state) + 1) % len(xesam.STATES)]
			value = 100
		
		self._state_count += 1
		
		if next_state_v == xesam.STATE_IDLE:
			return [xesam.STATE_IDLE, "0"]
		elif next_state_v == xesam.STATE_UPDATE  \
			 or next_state_v == xesam.STATE_FULL_INDEX:
			value -= 100/self.STATE_STEPS
			if value < 0:
				return next_state([next_state_v,"100"])
			return [next_state_v, str(value)]
		else:
			xesam.error ("Uknown state %s. Resetting to ['IDLE','0']" % next_state_v)
			return ["IDLE","0"]
	
